..
  This file is generated by setup.py

Read and write layered TIFF ImageSourceData and ImageResources tags
===================================================================

Psdtags is a Python library to read and write the Adobe Photoshop(r) specific
ImageResources (#34377) and ImageSourceData (#37724) TIFF tags, which contain
image resource blocks, layer and mask information found in a typical layered
TIFF file created by Photoshop.

The format is specified in the
`Adobe Photoshop TIFF Technical Notes (March 22, 2002)
<https://download.osgeo.org/libtiff/doc/TIFFphotoshop.pdf>`_
and
`Adobe Photoshop File Formats Specification (November 2019)
<https://www.adobe.com/devnet-apps/photoshop/fileformatashtml/>`_.

Adobe Photoshop is a registered trademark of Adobe Systems Inc.

:Author: `Christoph Gohlke <https://www.cgohlke.com>`_
:License: BSD-3-Clause
:Version: 2026.1.8
:DOI: `10.5281/zenodo.7879187 <https://doi.org/10.5281/zenodo.7879187>`_

Quickstart
----------

Install the psdtags package and all dependencies from the
`Python Package Index <https://pypi.org/project/psdtags/>`_::

    python -m pip install -U psdtags[all]

View the layer image and metadata stored in a layered TIFF file::

    python -m psdtags file.tif

See `Examples`_ for using the programming interface.

Source code, examples, and support are available on
`GitHub <https://github.com/cgohlke/psdtags>`_.

Requirements
------------

This revision was tested with the following requirements and dependencies
(other versions may work):

- `CPython <https://www.python.org>`_ 3.11.9, 3.12.10, 3.13.11, 3.14.2 64-bit
- `NumPy <https://pypi.org/project/numpy/>`_ 2.4.0
- `Imagecodecs <https://pypi.org/project/imagecodecs/>`_ 2026.1.1
  (required for compressing/decompressing image data)
- `Tifffile <https://pypi.org/project/tifffile/>`_ 2025.12.20
  (required for reading/writing tags from/to TIFF files)
- `Matplotlib <https://pypi.org/project/matplotlib/>`_ 3.10.8
  (required for plotting)

Revisions
---------

2026.x.x

- Improve code quality.

2025.12.12

- Make boolean and optional parameters keyword-only (breaking).

2025.9.19

- Write MTrn key before layers (#17).

2025.9.15

- Add CAI, GENI, and OCIO keys.
- Drop support for Python 3.10.

2025.5.10

- Support Python 3.14.

2025.1.1

- …

Refer to the CHANGES file for older revisions.

Notes
-----

The API is not stable yet and might change between revisions.

This library has been tested with a limited number of files only.

Additional layer information is not yet supported.

Consider `psd-tools <https://github.com/psd-tools/psd-tools>`_ and
`pytoshop <https://github.com/mdboom/pytoshop>`_  for working with
Adobe Photoshop PSD files.

Layered TIFF files can be read or written by Photoshop, Affinity Photo, and
Krita.

See also `Reading and writing a Photoshop TIFF
<https://www.amyspark.me/blog/posts/2021/11/14/reading-and-writing-tiff-psds.html>`_.

Examples
--------

Read the ImageSourceData tag value from a layered TIFF file and iterate over
all the channels:

.. code-block:: python

    >>> isd = TiffImageSourceData.fromtiff('layered.tif')
    >>> for layer in isd.layers:
    ...     layer.name
    ...     for channel in layer.channels:
    ...         ch = channel.data  # a numpy array
    ...
    'Background'
    'Reflect1'
    'Reflect2'
    'image'
    'Layer 1'
    'ORight'
    'I'
    'IShadow'
    'O'

Read the ImageResources tag value from the TIFF file, iterate over the blocks,
and get the thumbnail image:

.. code-block:: python

    >>> res = TiffImageResources.fromtiff('layered.tif')
    >>> for block in res.blocks:
    ...     blockname = block.name
    ...
    >>> res.thumbnail().shape
    (90, 160, 3)

Write the image, ImageSourceData and ImageResources to a new layered TIFF file:

.. code-block:: python

    >>> from tifffile import imread, imwrite
    >>> image = imread('layered.tif')
    >>> imwrite(
    ...     '_layered.tif',
    ...     image,
    ...     byteorder=isd.byteorder,  # must match ImageSourceData
    ...     photometric='rgb',  # must match ImageSourceData
    ...     metadata=None,  # do not write any tifffile specific metadata
    ...     extratags=[isd.tifftag(maxworkers=4), res.tifftag()],
    ... )

Verify that the new layered TIFF file contains readable ImageSourceData:

.. code-block:: python

    >>> assert isd == TiffImageSourceData.fromtiff('_layered.tif')
    >>> assert res == TiffImageResources.fromtiff('_layered.tif')

View the layer and mask information as well as the image resource blocks in
a layered TIFF file from a command line::

    python -m psdtags layered.tif

Refer to the `layered_tiff.py` example in the source distribution for
creating a layered TIFF file from individual layer images.